// -*- c -*-
// $Id: kstat.i,v 1.7 2001-03-13 09:28:44-06 annis Exp $
// $Source: /u/annis/code/NewMom/kstat/RCS/kstat.i,v $

// Copyright (c) 1999-2001 William S. Annis.  All rights reserved.
// This is free software; you can redistribute it and/or modify it
// under the same terms as Perl (the Artistic Licence).  Developed at
// the Department of Biostatistics and Medical Informatics, University
// of Wisconsin, Madison.

// With additions from Rick.Nooner@Level3.com which fix some serious
// memory leak problems.


%module wkstat

%{

#include "wkstat.h"

/*
  Add key,value pair to a dictionary.  Since it uses Py_BuildValue for the
  key/value pairs and we don't know the type of the value, we must use
  a macro for this to work.

  The reference count must be decremented for both the key and the value
  or memory leaks happen...

  Requires working variables PyObject *key, *value in same scope to work
  correctly.
*/
 
#define AddDict(dict, skey, itype, svalue) \
  do { \
    key = Py_BuildValue("s", skey); \
    value = Py_BuildValue(itype, svalue); \
    PyDict_SetItem(dict, key, value); \
    Py_DECREF(value); \
    Py_DECREF(key); \
  } while (0);

/*
  Special handling for unsigned numbers

  Requires working variables PyObject *key, *value in same scope to work
  correctly.
*/
#define AddDictUnsigned(dict, skey, svalue) \
  do { \
    key = Py_BuildValue("s", skey); \
    value = PyLong_FromUnsignedLong((unsigned long)svalue); \
    PyDict_SetItem(dict, key, value); \
    Py_DECREF(value); \
    Py_DECREF(key); \
  } while (0);

/*
  An encapsulation macro for adding a tuple value.
  In this case, we don't want to decrement the reference count.
*/
#define AddTuple(tuple, ref, itype, svalue) \
  do { \
    value = Py_BuildValue(itype, svalue); \
    PyTuple_SetItem(tuple, ref, value); \
  } while (0);

%}

// From kstat.h
typedef int kid_t;

// Turn this into tuple of (name, type, value).
// Since it's not clear that Python wants to cope with unsigned ints,
// I use unsigned longs for those.
%typemap(python, out) kstat_named_t * {
  PyObject *value;

  if ($source == NULL)
    {
        $target = Py_BuildValue("i", 0);
    }
  else
    {
      $target = PyTuple_New(3);
      AddTuple($target, 0, "s", $source->name);
      AddTuple($target, 1, "i", (int)$source->data_type);

      switch($source->data_type) {
      case KSTAT_DATA_CHAR:
        AddTuple($target, 2, "s", $source->value.c);
        break;
        
      case KSTAT_DATA_INT32:
        AddTuple($target, 2, "i", (int)$source->value.i32);
        break;
        
      case KSTAT_DATA_UINT32:
        AddTuple($target, 2, "l", $source->value.ui32);
        break;
        
      case KSTAT_DATA_INT64:
        AddTuple($target, 2, "l", $source->value.i64);
        break;
        
      case KSTAT_DATA_UINT64:
        AddTuple($target, 2, "l", $source->value.ui64);
        break;
      }
    }
}

// This is a more complex structure... turn it into a dictionary.
%typemap(python, out) kstat_io_t * {
  PyObject *key,*value;         /* used by AddDictUnsigned macro */
  $target = PyDict_New();

  AddDictUnsigned($target, "nread", $source->nread);
  AddDictUnsigned($target, "nwritten", $source->nwritten);
  AddDictUnsigned($target, "reads", $source->reads);
  AddDictUnsigned($target, "writes", $source->writes);

  /* Is this the right conversion for hrtime_t?  From double cast better? */
  AddDictUnsigned($target, "wtime", $source->wtime);
  AddDictUnsigned($target, "wlentime", $source->wlentime);
  AddDictUnsigned($target, "wlastupdate", $source->wlastupdate);
  AddDictUnsigned($target, "rtime", $source->rtime);
  AddDictUnsigned($target, "rlentime", $source->rlentime);
  AddDictUnsigned($target, "rlastupdate", $source->rlastupdate);
  AddDictUnsigned($target, "wcnt", $source->wcnt);
  AddDictUnsigned($target, "rcnt", $source->rcnt);
}

// Alas, more structs needed...
%typemap(python, out) vminfo_t * {
  PyObject *key, *value;        /* used by AddDictUnsigned macro */
  $target = PyDict_New();

  AddDictUnsigned($target, "freemem", $source->freemem);
  AddDictUnsigned($target, "swap_resv", $source->swap_resv);
  AddDictUnsigned($target, "swap_alloc", $source->swap_alloc);
  AddDictUnsigned($target, "swap_avail", $source->swap_avail);
  AddDictUnsigned($target, "swap_free", $source->swap_free);
}

%typemap(python, out) sysinfo_t * {
  PyObject *key, *value;        /* used by AddDictUnsigned macro */
  $target = PyDict_New();

  AddDictUnsigned($target, "updates", $source->updates);
  AddDictUnsigned($target, "runque", $source->runque);
  AddDictUnsigned($target, "runocc", $source->runocc);
  AddDictUnsigned($target, "swpque", $source->swpque);
  AddDictUnsigned($target, "swpocc", $source->swpocc);
  AddDictUnsigned($target, "waiting", $source->waiting);
}

// Now, this one is a royal pain in the parts!  There is a *lot* of
// information lurking here.  For now, I'm just going to extract all
// of it into a single dictionary, although several structs are
// involved here...
// v. /usr/include/sys/sysinfo.h for comments on the structs.

//typedef struct cpu_stat {
//	kmutex_t	cpu_stat_lock;
//	cpu_sysinfo_t	cpu_sysinfo;
//	cpu_syswait_t	cpu_syswait;
//	cpu_vminfo_t	cpu_vminfo;
//} cpu_stat_t;

%typemap(python, out) cpu_stat_t * {
  PyObject *key, *value;        /* used by AddDictUnsigned macro */
  $target = PyDict_New();

  // First, the data from cpu_sysinfo
  AddDictUnsigned($target, "cpu_user", $source->cpu_sysinfo.cpu[CPU_USER]);
  AddDictUnsigned($target, "cpu_kernel", $source->cpu_sysinfo.cpu[CPU_KERNEL]);
  AddDictUnsigned($target, "cpu_wait", $source->cpu_sysinfo.cpu[CPU_WAIT]);
  AddDictUnsigned($target, "cpu_idle", $source->cpu_sysinfo.cpu[CPU_IDLE]);
 
  AddDictUnsigned($target, "bread", $source->cpu_sysinfo.bread);
  AddDictUnsigned($target, "bwrite", $source->cpu_sysinfo.bwrite);
  AddDictUnsigned($target, "lread", $source->cpu_sysinfo.lread);
  AddDictUnsigned($target, "lwrite", $source->cpu_sysinfo.lwrite);
  AddDictUnsigned($target, "phread", $source->cpu_sysinfo.phread);
  AddDictUnsigned($target, "phwrite", $source->cpu_sysinfo.phwrite);
  AddDictUnsigned($target, "trap", $source->cpu_sysinfo.trap);
  AddDictUnsigned($target, "intr", $source->cpu_sysinfo.intr);
  AddDictUnsigned($target, "syscall", $source->cpu_sysinfo.syscall);
  AddDictUnsigned($target, "sysread", $source->cpu_sysinfo.sysread);
 
  AddDictUnsigned($target, "syswrite", $source->cpu_sysinfo.syswrite);
  AddDictUnsigned($target, "sysfork", $source->cpu_sysinfo.sysfork);
  AddDictUnsigned($target, "sysvfork", $source->cpu_sysinfo.sysvfork);
  AddDictUnsigned($target, "sysexec", $source->cpu_sysinfo.sysexec);
  AddDictUnsigned($target, "readch", $source->cpu_sysinfo.readch);
  AddDictUnsigned($target, "writech", $source->cpu_sysinfo.writech);
  AddDictUnsigned($target, "pswitch", $source->cpu_sysinfo.pswitch);
}

// DNLC stats.
%typemap(python, out) struct ncstats * {
  PyObject *key, *value;        /* used by AddDict macro */
  $target = PyDict_New();

  AddDict($target, "hits", "i", $source->hits);
  AddDict($target, "misses", "i", $source->misses);
  AddDict($target, "enters", "i", $source->enters);
  AddDict($target, "dbl_enters", "i", $source->dbl_enters);
  AddDict($target, "long_enter", "i", $source->long_enter);
  AddDict($target, "long_look", "i", $source->long_look);
  AddDict($target, "move_to_front", "i", $source->move_to_front);
  AddDict($target, "purges", "i", $source->purges);
}


%include wkstat.h

// Get these into my namespace...
#define WKSTAT_DATA_CHAR	KSTAT_DATA_CHAR
#define WKSTAT_DATA_INT32	KSTAT_DATA_INT32
#define WKSTAT_DATA_UINT32	KSTAT_DATA_UINT32
#define WKSTAT_DATA_INT64	KSTAT_DATA_INT64
#define WKSTAT_DATA_UINT64	KSTAT_DATA_UINT64

#define WKSTAT_TYPE_RAW		KSTAT_TYPE_RAW
#define WKSTAT_TYPE_NAMED	KSTAT_TYPE_NAMED
#define WKSTAT_TYPE_INTR	KSTAT_TYPE_INTR
#define WKSTAT_TYPE_IO		KSTAT_TYPE_IO
#define WKSTAT_TYPE_TIMER	KSTAT_TYPE_TIMER
