// -*- c -*-
// $Id: utmpx.i,v 1.2 1999-11-24 09:41:55-06 annis Exp annis $
// $Source: /u/annis/code/NewMom/utmp/RCS/utmpx.i,v $
//
// Copyright (c) 1999-2001 William S. Annis.  All rights reserved.
// This is free software; you can redistribute it and/or modify it
// under the same terms as Perl (the Artistic Licence).  Developed at
// the Department of Biostatistics and Medical Informatics, University
// of Wisconsin, Madison.
//
// The AddDict macro idea inspired by Rick Nooner at Level3.com, originally
// for the PyKstat library.

%module utmpx

%{

#include <utmp.h>
#include <utmpx.h>
#include <sys/time.h>

/*
  These were determined by gazing at man pages and headers.
*/
char *uttypes[] =
  {
    "EMPTY",
    "RUN_LVL",
    "BOOT_TIME",
    "OLD_TIME",
    "NEW_TIME",
    "INIT_PROCESS",
    "LOGIN_PROCESS",
    "USER_PROCESS",
    "DEAD_PROCESS",
    "ACCOUNTING"
  };


/*
  Add key,value pair to a dictionary.  Since it uses Py_BuildValue for the
  key/value pairs and we don't know the type of the value, we must use
  a macro for this to work.

  The reference count must be decremented for both the key and the value
  or memory leaks happen...

  Requires working variables PyObject *key, *value in same scope to work
  correctly.
*/
#define AddDict(dict, skey, itype, svalue) \
  do { \
    key = Py_BuildValue("s", skey); \
    value = Py_BuildValue(itype, svalue); \
    PyDict_SetItem(dict, key, value); \
    Py_DECREF(value); \
    Py_DECREF(key); \
  } while (0);

%}


%typemap(python, out) struct utmpx * {
  PyObject *key, *value;
  
  if ($source == NULL)
    {
        $target = Py_BuildValue("i", 0);
    }
  else
    {
      float tv;
      $target = PyDict_New();

      AddDict($target, "ut_user", "s", $source->ut_user);
      AddDict($target, "ut_id", "s", $source->ut_id);
      AddDict($target, "ut_line", "s", $source->ut_line);
      AddDict($target, "ut_pid", "i", $source->ut_pid);
      AddDict($target, "ut_type", "s", uttypes[$source->ut_type]);
      
      /* Python time (seconds + micros) */
      tv = $source->ut_tv.tv_sec + ($source->ut_tv.tv_usec / 1000.0);
      AddDict($target, "ut_tv", "f", tv);
      AddDict($target, "ut_session", "l", $source->ut_session);
      AddDict($target, "ut_syslen", "i", $source->ut_syslen);
      AddDict($target, "ut_host", "s", $source->ut_host);
      AddDict($target, "e_termination", "i", $source->ut_exit.e_termination);
      AddDict($target, "e_exit", "i", $source->ut_exit.e_exit);
    }
}


struct utmpx *getutxent(void);
void setutxent(void);
void endutxent(void);
int utmpxname(const char *);

// EOF
